#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from ovsdbapp import api


class API(api.API, metaclass=abc.ABCMeta):
    """An API based off of the vtep-ctl CLI interface

    This API basically mirrors the vtep-ctl operations with these changes:
    1. Methods that create objects will return a read-only view of the object
    2. Methods which list objects will return a list of read-only view objects
    """

    @abc.abstractmethod
    def add_ps(self, pswitch, may_exist=False, **columns):
        """Create a physical switch named 'pswitch'

        :param pswitch:   The name of the switch
        :type pswitch:    string or uuid.UUID
        :param may_exist: If True, don't fail if the switch already exists
        :type may_exist:  boolean
        :param columns:   Additional columns to directly set on the switch
        :returns:         :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def del_ps(self, pswitch, if_exists=False):
        """Delete physical switch 'pswitch' and all its ports

        :param pswitch:  The name or uuid of the switch
        :type pswitch:   string or uuid.UUID
        :type if_exists: If True, don't fail if the switch doesn't exist
        :type if_exists: boolean
        :returns:        :class:`Command` with no result
        """

    @abc.abstractmethod
    def list_ps(self):
        """Get all physical switches

        :returns: :class:`Command` with RowView list result
        """

    @abc.abstractmethod
    def get_ps(self, pswitch):
        """Get physical switch for 'pswitch'

        :param pswitch: The name of the pswitch
        :type pswitch:  string or uuid.UUID
        :returns:       :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def add_port(self, pswitch, port, may_exist=False):
        """Add a port named 'port' to physical switch named 'pswitch'

        :param pswitch:   The name of the switch
        :type pswitch:    string or uuid.UUID
        :param port:      The name of the port
        :type port:       string or uuid.UUID
        :param may_exist: If True, don't fail if the port already exists in
                          physical switch
        :type may_exist:  boolean
        :returns:         :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def del_port(self, pswitch, port, if_exists=False):
        """Delete a port named 'port' from physical switch named 'pswitch'

        :param pswitch:  The name or uuid of the switch
        :type pswitch:   string or uuid.UUID
        :param port:     The name of the port
        :type port:      string or uuid.UUID
        :type if_exists: If True, don't fail if the switch doesn't exist in
                         physical switch
        :type if_exists: boolean
        :returns:        :class:`Command` with no result
        """

    @abc.abstractmethod
    def list_ports(self, pswitch):
        """Get all ports of physical switch 'pswitch'

        :param pswitch: The name of the pswitch
        :type pswitch:  string or uuid.UUID
        :returns:       :class:`Command` with RowView list result
        """

    @abc.abstractmethod
    def get_port(self, port):
        """Get physical port for 'port'

        :param port: The name of the port
        :type port:  string or uuid.UUID
        :returns:    :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def add_ls(self, switch, may_exist=False, **columns):
        """Create a logical switch named 'switch'

        :param switch:    The name of the switch
        :type switch:     string or uuid.UUID
        :param may_exist: If True, don't fail if the switch already exists
        :type may_exist:  boolean
        :param columns:   Additional columns to directly set on the switch
        :returns:         :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def del_ls(self, switch, if_exists=False):
        """Delete logical switch 'switch' and all its ports

        :param switch:   The name or uuid of the switch
        :type switch:    string or uuid.UUID
        :type if_exists: If True, don't fail if the switch doesn't exist
        :type if_exists: boolean
        :returns:        :class:`Command` with no result
        """

    @abc.abstractmethod
    def list_ls(self):
        """Get all logical switches

        :returns: :class:`Command` with RowView list result
        """

    @abc.abstractmethod
    def get_ls(self, switch):
        """Get logical switch for 'switch'

        :returns: :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def bind_ls(self, pswitch, port, vlan, switch):
        """Bind 'switch' to 'pswitch'

        Bind logical switch to the port/vlan combination on the
        physical switch 'pswitch'.

        :param pswitch: The name or uuid of the physical switch
        :type pswitch:  string or uuid.UUID
        :param port:    name of port
        :type port:     string
        :param vlan:    number of VLAN
        :type vlan:     int
        :param switch:  The name or uuid of the switch
        :type switch:   string or uuid.UUID
        :returns:       :class:`Command` with no result
        """

    @abc.abstractmethod
    def unbind_ls(self, pswitch, port, vlan):
        """Unbind 'switch' from 'pswitch'

        Remove the logical switch binding from the port/vlan combination on
        the  physical switch pswitch.

        :param pswitch:  The name or uuid of the physical switch
        :type pswitch:   string or uuid.UUID
        :param port:     name of port
        :type port:      string
        :param vlan:     number of VLAN
        :type vlan:      int
        :param switch:   The name or uuid of the switch
        :type switch:    string or uuid.UUID
        :returns:        :class:`Command` with no result
        """

    @abc.abstractmethod
    def list_local_macs(self, switch):
        """Get all local MACs for 'switch'

        :param switch: The name of the switch
        :type switch:  string or uuid.UUID
        :returns:      :class:`Command` with list of RowView lists result.
                       First list contains 'Ucast_Macs_Local' table records,
                       second list contains 'Mcast_Macs_Local' table records.
        """

    @abc.abstractmethod
    def list_remote_macs(self, switch):
        """Get all remote MACs for 'switch'

        :param switch: The name of the switch
        :type switch:  string or uuid.UUID
        :returns:      :class:`Command` with list of RowView lists result.
                       First list contains 'Ucast_Macs_Remote' table records,
                       second list contains 'Mcast_Macs_Remote' table records.
        """

    @abc.abstractmethod
    def clear_local_macs(self, switch):
        """Clear the local MAC bindings for 'switch'

        :param switch: The name of the switch
        :type switch:  string or uuid.UUID
        :returns:      :class:`Command` with RowView result
        """

    @abc.abstractmethod
    def clear_remote_macs(self, switch):
        """Clear the remote MAC bindings for 'switch'

        :param switch: The name of the switch
        :type switch:  string or uuid.UUID
        :returns:      :class:`Command` with RowView result
        """



# -*- coding: utf-8 -*-
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
# sys.path.insert(0, os.path.abspath('.'))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'openstackdocstheme',
    'reno.sphinxext',
]

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/ovsdbapp'
openstackdocs_auto_name = False
openstackdocs_bug_project = 'ovsdbapp'
openstackdocs_bug_tag = ''

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'ovsdbapp Release Notes'
copyright = '2016, OpenStack Foundation'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
# The full version, including alpha/beta/rc tags.
release = ''
# The short X.Y version.
version = ''

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'openstackdocs'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
# html_extra_path = []

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'ovsdbappReleaseNotesdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    ('index', 'ovsdbappNotes.tex', 'ovsdbapp Release Notes Documentation',
     'ovsdbapp Developers', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for Internationalization output ------------------------------
locale_dirs = ['locale/']



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ovsdbapp.tests.functional import base
from ovsdbapp.tests.functional.schema.ovn_ic_northbound import fixtures
from ovsdbapp.tests import utils


class OvnIcNorthboundTest(base.FunctionalTestCase):
    schemas = ['OVN_IC_Northbound']
    fixture_class = base.venv.OvsOvnIcVenvFixture

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.schema_map = cls.schema_map.copy()
        cls.schema_map['OVN_IC_Northbound'] = cls.ovsvenv.ovn_icnb_connection

    def setUp(self):
        if not self.ovsvenv.has_icnb():
            self.skipTest("Installed version of OVN does not support ICNB")
        super(OvnIcNorthboundTest, self).setUp()
        self.api = self.useFixture(
            fixtures.IcNbApiFixture(self.connection)).obj
        self.table = self.api.tables['Transit_Switch']

    def _ts_add(self, *args, **kwargs):
        fix = self.useFixture(fixtures.TransitSwitchesFixture(self.api, *args,
                                                              **kwargs))
        self.assertIn(fix.obj.uuid, self.table.rows)
        return fix.obj

    def _test_ts_get(self, col):
        ts = self._ts_add(switch=utils.get_rand_device_name())
        val = getattr(ts, col)
        found = self.api.ts_get(val).execute(check_error=True)
        self.assertEqual(ts, found)

    def test_ts_get_uuid(self):
        self._test_ts_get('uuid')

    def test_ts_get_name(self):
        self._test_ts_get('name')

    def test_ts_add_name(self):
        name = utils.get_rand_device_name()
        ts = self._ts_add(name)
        self.assertEqual(name, ts.name)

    def test_ts_add_existing(self):
        name = utils.get_rand_device_name()
        self._ts_add(name)
        cmd = self.api.ts_add(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_ts_add_may_exist(self):
        name = utils.get_rand_device_name()
        sw = self._ts_add(name)
        sw2 = self.api.ts_add(name, may_exist=True).execute(check_error=True)
        self.assertEqual(sw, sw2)

    def test_ts_add_columns(self):
        external_ids = {'mykey': 'myvalue', 'yourkey': 'yourvalue'}
        ts = self._ts_add(switch=utils.get_rand_device_name(),
                          external_ids=external_ids)
        self.assertEqual(external_ids, ts.external_ids)

    def test_ts_del(self):
        sw = self._ts_add(switch=utils.get_rand_device_name())
        self.api.ts_del(sw.uuid).execute(check_error=True)
        self.assertNotIn(sw.uuid, self.table.rows)

    def test_ts_del_by_name(self):
        name = utils.get_rand_device_name()
        self._ts_add(name)
        self.api.ts_del(name).execute(check_error=True)
        found = self.api.ts_get(name).execute()
        self.assertIsNone(found)

    def test_ts_del_no_existing(self):
        name = utils.get_rand_device_name()
        cmd = self.api.ts_del(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_ts_del_if_exists(self):
        name = utils.get_rand_device_name()
        self.api.ts_del(name, if_exists=True).execute(check_error=True)
        found = self.api.ts_get(name).execute()
        self.assertIsNone(found)

    def test_ts_list(self):
        switches = {self._ts_add(str(i)) for i in range(3)}
        switch_set = set(self.api.ts_list().execute(check_error=True))
        self.assertTrue(switches.issubset(switch_set))



# Copyright (c) 2017 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from ovsdbapp import event
from ovsdbapp.tests import base


class TestEvent(event.RowEvent):
    def __init__(self, events=(event.RowEvent.ROW_CREATE,),
                 table="FakeTable", conditions=(("col", "=", "val"),),
                 priority=None):
        super().__init__(events, table, conditions)
        if priority is not None:
            self.priority = priority

    def run(self):
        pass

    def matches(self):
        pass


class OtherTestEvent(TestEvent):
    pass


class TestRowEvent(base.TestCase):
    def test_compare_stop_event(self):
        r = TestEvent()
        self.assertNotEqual((r, "fake", "fake", "fake"), event.STOP_EVENT)

    def test_compare_equality(self):
        self.assertEqual(TestEvent(), TestEvent())
        self.assertNotEqual(TestEvent(), TestEvent(table="NotFaketable"))
        self.assertNotEqual(TestEvent(), TestEvent(conditions=None))
        self.assertNotEqual(TestEvent(priority=1), TestEvent(priority=2))
        self.assertNotEqual(TestEvent(), OtherTestEvent())


class TestRowEventHandler(base.TestCase):
    def setUp(self):
        super().setUp()
        self.handler = event.RowEventHandler()
        self.assertEqual(0, len(tuple(self.handler._watched_events)))

    def test_watch_event(self):
        event = TestEvent()
        expected = (event,)
        self.handler.watch_event(event)
        self.assertCountEqual(expected, self.handler._watched_events)
        return expected

    def test_watch_events(self):
        events = [TestEvent(priority=r) for r in range(10)]
        expected = list(reversed(events))
        self.handler.watch_events(events)
        self.assertEqual(expected, list(self.handler._watched_events))
        return expected

    def test_unwatch_event(self):
        expected = self.test_watch_events()
        removed = expected.pop(5)
        self.handler.unwatch_event(removed)
        self.assertEqual(expected, list(self.handler._watched_events))

    def test_unwatch_events(self):
        expected = self.test_watch_events()
        removed = [expected.pop(5), expected.pop(7)]
        self.handler.unwatch_events(removed)
        self.assertEqual(expected, list(self.handler._watched_events))

    def test_add_duplicate(self):
        self.handler.watch_event(TestEvent())
        self.handler.watch_event(TestEvent())
        self.assertCountEqual(self.handler._watched_events, [TestEvent()])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures


class ImplIdlFixture(fixtures.Fixture):
    create, delete = (None, None)
    delete_args = {'if_exists': True}
    delete_id = 'uuid'

    def __init__(self, api, *args, **kwargs):
        super(ImplIdlFixture, self).__init__()
        self.api = api
        self.args = args
        self.kwargs = kwargs

    def _setUp(self):
        create_fn = getattr(self.api, self.create)
        delete_fn = getattr(self.api, self.delete)
        self.obj = create_fn(*self.args, **self.kwargs).execute(
            check_error=True)
        del_value = getattr(self.obj, self.delete_id)
        self.addCleanup(delete_fn(del_value,
                        **self.delete_args).execute, check_error=True)


class ApiImplFixture(fixtures.Fixture):
    api_cls = None

    def __init__(self, *args, **kwargs):
        super().__init__()
        self.args = args
        self.kwargs = kwargs
        # Create a unique subclass for every api_cls so that none of
        # them share an ovsdb_connection, a class attribute that
        # different tests running in the same process can trample on
        self.cls = type('Test' + self.api_cls.__name__, (self.api_cls,), {})

    def _setUp(self):
        self.obj = self.cls(*self.args, **self.kwargs)



# -*- coding: utf-8 -*-
# Copyright (c) 2017 Red Hat, Inc.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ovsdbapp.backend.ovs_idl import idlutils
from ovsdbapp.schema.hardware_vtep.commands import get_global_record
from ovsdbapp.tests.functional import base
from ovsdbapp.tests.functional.schema.hardware_vtep import fixtures
from ovsdbapp.tests import utils


class HardwareVtepTest(base.FunctionalTestCase):
    schemas = ["hardware_vtep"]
    fixture_class = base.venv.OvsVtepVenvFixture

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.schema_map = cls.schema_map.copy()
        cls.schema_map['hardware_vtep'] = cls.ovsvenv.ovs_connection

    def setUp(self):
        if not self.ovsvenv.has_vtep:
            self.skipTest("Installed version of OVS does not support VTEP")
        super().setUp()
        self.api = self.useFixture(
            fixtures.HwVtepApiFixture(self.connection)).obj


class TestPhysicalSwitchOps(HardwareVtepTest):
    def setUp(self):
        super().setUp()
        self.table = self.api.tables['Physical_Switch']
        self.config = get_global_record(self.api)

    def _add_ps(self, *args, **kwargs):
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(self.api, *args,
                                                            **kwargs)).obj
        self.assertIn(ps.uuid, self.table.rows)
        self.assertIn(ps, self.config.switches)
        return ps

    def _test_get_ps(self, col):
        ps = self._add_ps(pswitch=utils.get_rand_device_name())
        val = getattr(ps, col)
        found = self.api.get_ps(val).execute(check_error=True)
        self.assertEqual(ps, found)

    def test_get_ps_uuid(self):
        self._test_get_ps('uuid')

    def test_get_ps_name(self):
        self._test_get_ps('name')

    def test_add_ps_name(self):
        name = utils.get_rand_device_name()
        sw = self._add_ps(name)
        self.assertEqual(name, sw.name)

    def test_add_ps_exists(self):
        name = utils.get_rand_device_name()
        self._add_ps(name)
        cmd = self.api.add_ps(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_add_ps_may_exist(self):
        name = utils.get_rand_device_name()
        sw = self._add_ps(name)
        sw2 = self.api.add_ps(name, may_exist=True).execute(check_error=True)
        self.assertEqual(sw, sw2)

    def test_del_ps(self):
        name = utils.get_rand_device_name()
        sw = self._add_ps(name)
        self.api.del_ps(sw.uuid).execute(check_error=True)
        self.assertNotIn(sw.uuid, self.table.rows)
        self.assertNotIn(sw, self.config.switches)

    def test_del_ps_by_name(self):
        name = utils.get_rand_device_name()
        sw = self._add_ps(name)
        self.api.del_ps(name).execute(check_error=True)
        self.assertNotIn(sw.uuid, self.table.rows)
        self.assertNotIn(sw, self.config.switches)

    def test_del_ps_no_exist(self):
        name = utils.get_rand_device_name()
        cmd = self.api.del_ps(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_del_ps_if_exists(self):
        name = utils.get_rand_device_name()
        self.api.del_ps(name, if_exists=True).execute(check_error=True)

    def test_list_ps(self):
        switches = {self._add_ps(str(i)) for i in range(3)}
        switch_set = set(self.api.list_ps().execute(check_error=True))
        self.assertTrue(switches.issubset(switch_set))


class TestPhysicalPortOps(HardwareVtepTest):
    def setUp(self):
        super().setUp()
        self.table = self.api.tables['Physical_Port']
        self.ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, utils.get_rand_device_name())).obj

    def _add_port(self, *args, name=None, **kwargs):
        port = self.api.add_port(self.ps.uuid,
                                 name or utils.get_rand_device_name(),
                                 *args, **kwargs).execute()
        self.assertIn(port.uuid, self.table.rows)
        self.assertIn(port, self.ps.ports)
        return port

    def _test_get_port(self, col):
        port = self._add_port(name=utils.get_rand_device_name())
        val = getattr(port, col)
        found = self.api.get_port(val).execute(check_error=True)
        self.assertEqual(port, found)

    def test_get_port_uuid(self):
        self._test_get_port('uuid')

    def test_get_port_name(self):
        self._test_get_port('name')

    def test_add_port_name(self):
        name = utils.get_rand_device_name()
        port = self._add_port(name=name)
        self.assertEqual(name, port.name)

    def test_add_port_exists(self):
        name = utils.get_rand_device_name()
        self._add_port(name=name)
        cmd = self.api.add_port(self.ps.uuid, name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_add_port_may_exist(self):
        name = utils.get_rand_device_name()
        port = self._add_port(name=name)
        port2 = self.api.add_port(
            self.ps.uuid, name, may_exist=True).execute(check_error=True)
        self.assertEqual(port, port2)

    def test_del_port(self):
        port = self._add_port()
        self.api.del_port(self.ps.name, port.name).execute(check_error=True)
        self.assertNotIn(port.uuid, self.table.rows)
        self.assertNotIn(port, self.ps.ports)

    def test_del_port_by_name(self):
        name = utils.get_rand_device_name()
        port = self._add_port(name=name)
        self.api.del_port(self.ps.uuid, name).execute(check_error=True)
        self.assertNotIn(port.uuid, self.table.rows)
        self.assertNotIn(port, self.ps.ports)

    def test_del_port_no_exist(self):
        name = utils.get_rand_device_name()
        cmd = self.api.del_port(self.ps.uuid, name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_del_ps_if_exists(self):
        name = utils.get_rand_device_name()
        self.api.del_port(self.ps.uuid, name, if_exists=True).execute(
            check_error=True)

    def test_list_ps(self):
        ports = {self._add_port(str(i)) for i in range(3)}
        port_set = set(self.api.list_ports(
            self.ps.uuid).execute(check_error=True))
        self.assertTrue(ports.issubset(port_set))


class TestLogicalSwitchOps(HardwareVtepTest):
    def setUp(self):
        super().setUp()
        self.table = self.api.tables['Logical_Switch']
        self.vlan = 10

    def _add_ls(self, *args, **kwargs):
        ls = self.useFixture(fixtures.LogicalSwitchFixture(self.api, *args,
                                                           **kwargs)).obj
        self.assertIn(ls.uuid, self.table.rows)
        return ls

    def _test_get_ls(self, col):
        ls = self._add_ls(switch=utils.get_rand_device_name())
        val = getattr(ls, col)
        found = self.api.get_ls(val).execute(check_error=True)
        self.assertEqual(ls, found)

    def test_get_ls_uuid(self):
        self._test_get_ls('uuid')

    def test_get_ls_name(self):
        self._test_get_ls('name')

    def test_add_ls_name(self):
        name = utils.get_rand_device_name()
        sw = self._add_ls(name)
        self.assertEqual(name, sw.name)

    def test_add_ls_exists(self):
        name = utils.get_rand_device_name()
        self._add_ls(name)
        cmd = self.api.add_ls(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_add_ls_may_exist(self):
        name = utils.get_rand_device_name()
        sw = self._add_ls(name)
        sw2 = self.api.add_ls(name, may_exist=True).execute(check_error=True)
        self.assertEqual(sw, sw2)

    def test_del_ls(self):
        name = utils.get_rand_device_name()
        sw = self._add_ls(name)
        self.api.del_ls(sw.uuid).execute(check_error=True)
        self.assertNotIn(sw.uuid, self.table.rows)

    def test_del_ls_by_name(self):
        name = utils.get_rand_device_name()
        self._add_ls(name)
        self.api.del_ls(name).execute(check_error=True)

    def test_del_ls_no_exist(self):
        name = utils.get_rand_device_name()
        cmd = self.api.del_ls(name)
        self.assertRaises(RuntimeError, cmd.execute, check_error=True)

    def test_del_ls_if_exists(self):
        name = utils.get_rand_device_name()
        self.api.del_ls(name, if_exists=True).execute(check_error=True)

    def test_list_ls(self):
        switches = {self._add_ls(str(i)) for i in range(3)}
        switch_set = set(self.api.list_ls().execute(check_error=True))
        self.assertTrue(switches.issubset(switch_set))

    def test_bind_unbind_ls(self):
        name = utils.get_rand_device_name()
        switch = self._add_ls(name)
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, name)).obj
        port = self.api.add_port(ps.uuid, name,
                                 may_exist=True).execute(check_error=True)

        self.api.bind_ls(ps.name, port.name,
                         self.vlan, switch.name).execute(check_error=True)
        self.assertEqual(port.vlan_bindings, {self.vlan: switch})

        self.api.unbind_ls(ps.name, port.name,
                           self.vlan).execute(check_error=True)
        self.assertEqual(port.vlan_bindings, {})

    def _test_bind_ls_no_exist(self,
                               pswitch_name=None,
                               port_name=None,
                               switch_name=None):
        cmd = self.api.bind_ls(pswitch_name, port_name, self.vlan, switch_name)
        self.assertRaises(idlutils.RowNotFound, cmd.execute, check_error=True)

    def test_bind_ls_pswitch_no_exist(self):
        name = utils.get_rand_device_name()
        switch = self._add_ls(utils.get_rand_device_name())
        self._test_bind_ls_no_exist(pswitch_name=name,
                                    port_name=name,
                                    switch_name=switch.name)

    def test_bind_ls_port_no_exist(self):
        name = utils.get_rand_device_name()
        switch = self._add_ls(utils.get_rand_device_name())
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, utils.get_rand_device_name())).obj
        self._test_bind_ls_no_exist(pswitch_name=ps.name,
                                    port_name=name,
                                    switch_name=switch.name)

    def test_bind_ls_switch_no_exist(self):
        name = utils.get_rand_device_name()
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, utils.get_rand_device_name())).obj
        port = self.api.add_port(ps.uuid, utils.get_rand_device_name(),
                                 may_exist=True).execute(check_error=True)
        self._test_bind_ls_no_exist(pswitch_name=ps.name,
                                    port_name=port.name,
                                    switch_name=name)

    def test_unbind_ls_no_exist(self):
        name = utils.get_rand_device_name()
        switch = self._add_ls(name)
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, name)).obj
        port = self.api.add_port(ps.uuid, name,
                                 may_exist=True).execute(check_error=True)
        self.api.bind_ls(ps.name, port.name,
                         self.vlan, switch.name).execute(check_error=True)

        for pswitch, port in [('pswitch', port.name), (ps.name, 'port')]:
            cmd = self.api.unbind_ls(pswitch, port, self.vlan)
            self.assertRaises(idlutils.RowNotFound, cmd.execute,
                              check_error=True)

    def test_unbind_ls_vlan_no_exists(self):
        name = utils.get_rand_device_name()
        switch = self._add_ls(name)
        ps = self.useFixture(fixtures.PhysicalSwitchFixture(
            self.api, name)).obj
        port = self.api.add_port(ps.uuid, name,
                                 may_exist=True).execute(check_error=True)
        self.api.bind_ls(ps.name, port.name,
                         self.vlan, switch.name).execute(check_error=True)
        vlan_bindings = port.vlan_bindings.copy()
        self.api.unbind_ls(ps.name, port.name,
                           self.vlan + 1).execute(check_error=True)
        self.assertEqual(port.vlan_bindings, vlan_bindings)


class TestMacBindingsOps(HardwareVtepTest):
    def setUp(self):
        super().setUp()
        self.ls = self.useFixture(fixtures.LogicalSwitchFixture(
            self.api, utils.get_rand_device_name())).obj
        self.mac = '0a:00:d0:af:20:c0'
        self.ip = '192.168.0.1'
        self.ovsvenv.call(['vtep-ctl', 'add-ucast-local',
                           self.ls.name, self.mac, self.ip])
        self.ovsvenv.call(['vtep-ctl', 'add-mcast-local',
                           self.ls.name, self.mac, self.ip])
        self.ovsvenv.call(['vtep-ctl', 'add-ucast-remote',
                           self.ls.name, self.mac, self.ip])
        self.ovsvenv.call(['vtep-ctl', 'add-mcast-remote',
                           self.ls.name, self.mac, self.ip])
        for args in [
            ['vtep-ctl', 'del-ucast-local', self.ls.name, self.mac],
            ['vtep-ctl', 'del-mcast-local', self.ls.name, self.mac, self.ip],
            ['vtep-ctl', 'del-ucast-remote', self.ls.name, self.mac],
            ['vtep-ctl', 'del-mcast-remote', self.ls.name, self.mac, self.ip]
        ]:
            self.addCleanup(self.ovsvenv.call, args)

    def test_list_local_macs(self):
        local_macs = self.api.list_local_macs(
            self.ls.name).execute(check_error=True)
        for macs in local_macs:
            self.assertEqual(len(macs), 1)
            self.assertEqual(macs[0].MAC, self.mac)

    def test_list_remote_macs(self):
        remote_macs = self.api.list_remote_macs(
            self.ls.name).execute(check_error=True)
        for macs in remote_macs:
            self.assertEqual(len(macs), 1)
            self.assertEqual(macs[0].MAC, self.mac)

    def test_clear_local_macs(self):
        ucast_table = self.api.tables['Ucast_Macs_Local']
        mcast_table = self.api.tables['Mcast_Macs_Local']
        for table in [ucast_table, mcast_table]:
            self.assertEqual(len(table.rows), 1)

        self.api.clear_local_macs(self.ls.name).execute(check_error=True)
        for table in [ucast_table, mcast_table]:
            self.assertEqual(len(table.rows), 0)

    def test_clear_remote_macs(self):
        ucast_table = self.api.tables['Ucast_Macs_Remote']
        mcast_table = self.api.tables['Mcast_Macs_Remote']
        for table in [ucast_table, mcast_table]:
            self.assertEqual(len(table.rows), 1)

        self.api.clear_remote_macs(self.ls.name).execute(check_error=True)
        for table in [ucast_table, mcast_table]:
            self.assertEqual(len(table.rows), 0)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ovsdbapp.backend import ovs_idl
from ovsdbapp.backend.ovs_idl import idlutils
from ovsdbapp.schema.hardware_vtep import api
from ovsdbapp.schema.hardware_vtep import commands as cmd


class HwVtepApiIdlImpl(ovs_idl.Backend, api.API):
    schema = 'hardware_vtep'
    lookup_table = {
        'Global': idlutils.RowLookup('Global', None, None),
        'Physical_Switch': idlutils.RowLookup('Physical_Switch', 'name', None),
        'Physical_Port': idlutils.RowLookup('Physical_Port', 'name', None),
        'Logical_Switch': idlutils.RowLookup('Logical_Switch', 'name', None),
        'Ucast_Macs_Local': idlutils.RowLookup('Ucast_Macs_Local', None, None),
        'Mcast_Macs_Local': idlutils.RowLookup('Mcast_Macs_Local', None, None),
        'Ucast_Macs_Remote': idlutils.RowLookup('Ucast_Macs_Remote',
                                                None, None),
        'Mcast_Macs_Remote': idlutils.RowLookup('Mcast_Macs_Remote',
                                                None, None),
    }

    def add_ps(self, pswitch, may_exist=False, **columns):
        return cmd.AddPsCommand(self, pswitch, may_exist, **columns)

    def del_ps(self, pswitch, if_exists=False):
        return cmd.DelPsCommand(self, pswitch, if_exists)

    def list_ps(self):
        return cmd.ListPsCommand(self)

    def get_ps(self, pswitch):
        return cmd.GetPsCommand(self, pswitch)

    def add_port(self, pswitch, port, may_exist=False):
        return cmd.AddPortCommand(self, pswitch, port, may_exist)

    def del_port(self, pswitch, port, if_exists=False):
        return cmd.DelPortCommand(self, pswitch, port, if_exists)

    def list_ports(self, pswitch):
        return cmd.ListPortsCommand(self, pswitch)

    def get_port(self, port):
        return cmd.GetPortCommand(self, port)

    def add_ls(self, switch, may_exist=False, **columns):
        return cmd.AddLsCommand(self, switch, may_exist, **columns)

    def del_ls(self, switch, if_exists=False):
        return cmd.DelLsCommand(self, switch, if_exists)

    def list_ls(self):
        return cmd.ListLsCommand(self)

    def get_ls(self, switch):
        return cmd.GetLsCommand(self, switch)

    def bind_ls(self, pswitch, port, vlan, switch):
        return cmd.BindLsCommand(self, pswitch, port, vlan, switch)

    def unbind_ls(self, pswitch, port, vlan):
        return cmd.UnbindLsCommand(self, pswitch, port, vlan)

    def clear_local_macs(self, switch):
        return cmd.ClearLocalMacsCommand(self, switch)

    def clear_remote_macs(self, switch):
        return cmd.ClearRemoteMacsCommand(self, switch)

    def list_local_macs(self, switch):
        return cmd.ListLocalMacsCommand(self, switch)

    def list_remote_macs(self, switch):
        return cmd.ListRemoteMacsCommand(self, switch)
